STEP_EVENT_FRAGMENTS = '''
fragment eventMetadataEntryFragment on EventMetadataEntry {
  __typename
  label
  description
  ... on EventPathMetadataEntry {
    path
  }
  ... on EventJsonMetadataEntry {
    jsonString
  }
  ... on EventUrlMetadataEntry {
    url
  }
  ... on EventTextMetadataEntry {
    text
  }
  ... on EventMarkdownMetadataEntry {
    mdStr
  }
  ... on EventPythonArtifactMetadataEntry {
    module
    name
  }
}

fragment errorFragment on PythonError {
  message
  stack
  className
  cause {
    message
    stack
    className
    cause {
      message
      stack
      className
    }
  }
}


fragment stepEventFragment on StepEvent {
  step {
    key
    inputs {
      name
      type {
        key
      }
      dependsOn {
        key
      }
    }
    outputs {
      name
      type {
        key
      }
    }
    solidHandleID
    kind
    metadata {
      key
      value
    }
  }
  ... on MessageEvent {
    runId
    message
    timestamp
    level
  }
  ... on StepExpectationResultEvent {
    expectationResult {
      success
      label
      description
      metadataEntries {
        ...eventMetadataEntryFragment
      }
    }
  }
  ... on StepMaterializationEvent {
    materialization {
      label
      description
      metadataEntries {
        ...eventMetadataEntryFragment
      }
    }
  }
  ... on ExecutionStepInputEvent {
    inputName
    typeCheck {
      __typename
      success
      label
      description
      metadataEntries {
        ...eventMetadataEntryFragment
      }
    }
  }
  ... on ExecutionStepOutputEvent {
    outputName
    typeCheck {
      __typename
      success
      label
      description
      metadataEntries {
        ...eventMetadataEntryFragment
      }
    }
  }
  ... on ExecutionStepFailureEvent {
    error {
      ...errorFragment
    }
    failureMetadata {
      label
      description
      metadataEntries {
        ...eventMetadataEntryFragment
      }
    }
  }
  ... on ExecutionStepUpForRetryEvent {
    retryError: error {
      ...errorFragment
    }
    secondsToWait
  }
  ... on EngineEvent {
    metadataEntries {
      ...eventMetadataEntryFragment
    }
    markerStart
    markerEnd
    engineError: error {
      ...errorFragment
    }
  }
}
'''

MESSAGE_EVENT_FRAGMENTS = (
    '''
fragment messageEventFragment on MessageEvent {
  runId
  message
  timestamp
  level
  ...stepEventFragment
  ... on PipelineInitFailureEvent {
    initError: error {
      ...errorFragment
    }
  }
}
'''
    + STEP_EVENT_FRAGMENTS
)


EXECUTE_RUN_IN_PROCESS_RESULT_FRAGMENT = '''
fragment executeRunInProcessResultFragment on ExecuteRunInProcessResult {
	__typename
	... on InvalidStepError {
		invalidStepKey
	}
	... on InvalidOutputError {
		stepKey
		invalidOutputName
	}
	... on PipelineConfigValidationInvalid {
    pipelineName
		errors {
			__typename
			message
			path
			reason
		}
	}
	... on PipelineNotFoundError {
		message
		pipelineName
	}
  ... on PythonError {
    message
    stack
  }
	... on ExecuteRunInProcessSuccess {
		run {
			runId
			status
			pipeline {
				name
			}
			runConfigYaml
			mode
		}
	}
}
'''

EXECUTE_RUN_IN_PROCESS_MUTATION = (
    '''
mutation(
  $repositoryLocationName: String!
  $repositoryName: String!
  $runId: String!
) {
  executeRunInProcess(
    repositoryLocationName: $repositoryLocationName
    repositoryName: $repositoryName
    runId: $runId
  ) {
    ...executeRunInProcessResultFragment
  }
}
'''
    + EXECUTE_RUN_IN_PROCESS_RESULT_FRAGMENT
)

EXECUTE_PLAN_MUTATION = (
    '''
mutation(
  $executionParams: ExecutionParams!
) {
  executePlan(
    executionParams: $executionParams,
  ) {
    __typename
    ... on InvalidStepError {
      invalidStepKey
    }
    ... on PipelineConfigValidationInvalid {
      pipelineName
      errors {
        __typename
        message
        path
        reason
      }
    }
    ... on PipelineNotFoundError {
      message
      pipelineName
    }
    ... on PythonError {
      message
      stack
    }
    ... on ExecutePlanSuccess {
      pipeline {
        name
      }
      hasFailures
      stepEvents {
        __typename
        ...stepEventFragment
      }
    }
  }
}
'''
    + STEP_EVENT_FRAGMENTS
)

RAW_EXECUTE_PLAN_MUTATION = '''
mutation(
  $executionParams: ExecutionParams!
) {
  executePlan(
    executionParams: $executionParams,
  ) {
    __typename
    ... on InvalidStepError {
      invalidStepKey
    }
    ... on PipelineConfigValidationInvalid {
      pipelineName
      errors {
        __typename
        message
        path
        reason
      }
    }
    ... on PipelineNotFoundError {
      message
      pipelineName
    }
    ... on PythonError {
      message
      stack
      cause {
          message
          stack
      }
    }
    ... on ExecutePlanSuccess {
      pipeline {
        name
      }
      hasFailures
      rawEventRecords
    }
  }
}
'''

SUBSCRIPTION_QUERY = (
    MESSAGE_EVENT_FRAGMENTS
    + '''
subscription subscribeTest($runId: ID!) {
    pipelineRunLogs(runId: $runId) {
        __typename
        ... on PipelineRunLogsSubscriptionSuccess {
            run {
              runId
            },
            messages {
                __typename
                ...messageEventFragment
            }
        }
        ... on PipelineRunLogsSubscriptionFailure {
            missingRunId
        }
    }
}
'''
)

LAUNCH_PIPELINE_EXECUTION_MUTATION = '''
mutation(
  $executionParams: ExecutionParams!
) {
  launchPipelineExecution(
    executionParams: $executionParams,
  ) {
    __typename
    ... on InvalidStepError {
      invalidStepKey
    }
    ... on InvalidOutputError {
      stepKey
      invalidOutputName
    }
    ... on PipelineConfigValidationInvalid {
      pipelineName
      errors {
        __typename
        message
        path
        reason
      }
    }
    ... on PipelineNotFoundError {
      message
      pipelineName
    }
    ... on PythonError {
      message
      stack
    }
    ... on LaunchPipelineRunSuccess {
      run {
        runId
        status
        pipeline {
          name
        }
        runConfigYaml
        mode
      }
    }
  }
}
'''


LAUNCH_PIPELINE_REEXECUTION_MUTATION = '''
mutation(
  $executionParams: ExecutionParams!
) {
  launchPipelineReexecution(
    executionParams: $executionParams,
  ) {
    __typename
    ... on InvalidStepError {
      invalidStepKey
    }
    ... on InvalidOutputError {
      stepKey
      invalidOutputName
    }
    ... on PipelineConfigValidationInvalid {
      pipelineName
      errors {
        __typename
        message
        path
        reason
      }
    }
    ... on PipelineNotFoundError {
      message
      pipelineName
    }
    ... on PythonError {
      message
      stack
    }
    ... on LaunchPipelineRunSuccess {
      run {
        runId
        status
        pipeline {
          name
        }
        runConfigYaml
        mode
      }
    }
  }
}
'''



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots['TestExecuteSchedule.test_tick_skip[sqlite_with_cli_api_run_launcher_in_process_env] 1'] = {
    'stats': {
        'ticksFailed': 0,
        'ticksSkipped': 1,
        'ticksStarted': 0,
        'ticksSucceeded': 0
    },
    'ticks': [
        {
            'status': 'SKIPPED',
            'tickId': '1'
        }
    ],
    'ticksCount': 1
}

snapshots['TestExecuteSchedule.test_tick_success[sqlite_with_cli_api_run_launcher_in_process_env] 1'] = {
    'stats': {
        'ticksFailed': 0,
        'ticksSkipped': 0,
        'ticksStarted': 0,
        'ticksSucceeded': 1
    },
    'ticks': [
        {
            'status': 'SUCCESS',
            'tickId': '1'
        }
    ],
    'ticksCount': 1
}

snapshots['TestExecuteSchedule.test_should_execute_scheduler_error[sqlite_with_cli_api_run_launcher_in_process_env] 1'] = {
    'stats': {
        'ticksFailed': 1,
        'ticksSkipped': 0,
        'ticksStarted': 0,
        'ticksSucceeded': 0
    },
    'ticks': [
        {
            'status': 'FAILURE',
            'tickId': '1'
        }
    ],
    'ticksCount': 1
}

snapshots['TestExecuteSchedule.test_tags_scheduler_error[sqlite_with_cli_api_run_launcher_in_process_env] 1'] = {
    'stats': {
        'ticksFailed': 0,
        'ticksSkipped': 0,
        'ticksStarted': 0,
        'ticksSucceeded': 1
    },
    'ticks': [
        {
            'status': 'SUCCESS',
            'tickId': '1'
        }
    ],
    'ticksCount': 1
}

snapshots['TestExecuteSchedule.test_environment_dict_scheduler_error[sqlite_with_cli_api_run_launcher_in_process_env] 1'] = {
    'stats': {
        'ticksFailed': 0,
        'ticksSkipped': 0,
        'ticksStarted': 0,
        'ticksSucceeded': 1
    },
    'ticks': [
        {
            'status': 'SUCCESS',
            'tickId': '1'
        }
    ],
    'ticksCount': 1
}

snapshots['TestExecuteSchedule.test_query_multiple_schedule_ticks[sqlite_with_cli_api_run_launcher_in_process_env] 1'] = [
    {
        'name': 'dynamic_config',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'environment_dict_error_schedule',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 1
            },
            'ticks': [
                {
                    'status': 'SUCCESS',
                    'tickId': '3'
                }
            ],
            'ticksCount': 1
        }
    },
    {
        'name': 'invalid_config_schedule',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'no_config_pipeline_hourly_schedule',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 1
            },
            'ticks': [
                {
                    'status': 'SUCCESS',
                    'tickId': '1'
                }
            ],
            'ticksCount': 1
        }
    },
    {
        'name': 'no_config_pipeline_hourly_schedule_with_config_fn',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'no_config_should_execute',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 1,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
                {
                    'status': 'SKIPPED',
                    'tickId': '2'
                }
            ],
            'ticksCount': 1
        }
    },
    {
        'name': 'partition_based',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'partition_based_custom_selector',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'partition_based_decorator',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'partition_based_multi_mode_decorator',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'should_execute_error_schedule',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'solid_selection_daily_decorator',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'solid_selection_hourly_decorator',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'solid_selection_monthly_decorator',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'solid_selection_weekly_decorator',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'tagged_pipeline_override_schedule',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'tagged_pipeline_schedule',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    },
    {
        'name': 'tags_error_schedule',
        'scheduleState': {
            'stats': {
                'ticksFailed': 0,
                'ticksSkipped': 0,
                'ticksStarted': 0,
                'ticksSucceeded': 0
            },
            'ticks': [
            ],
            'ticksCount': 0
        }
    }
]

snapshots['TestExecuteSchedule.test_invalid_config_schedule_error[sqlite_with_cli_api_run_launcher_in_process_env] 1'] = {
    'stats': {
        'ticksFailed': 0,
        'ticksSkipped': 0,
        'ticksStarted': 0,
        'ticksSucceeded': 1
    },
    'ticks': [
        {
            'status': 'SUCCESS',
            'tickId': '1'
        }
    ],
    'ticksCount': 1
}



import re
import sys

import pytest
from dagster_graphql.test.utils import execute_dagster_graphql, infer_pipeline_selector

from .graphql_context_test_suite import LegacyReadonlyGraphQLContextTestMatrix

SCHEMA_OR_ERROR_SUBSET_QUERY = '''
query EnvironmentQuery($selector: PipelineSelector!){
    runConfigSchemaOrError(selector: $selector) {
        __typename
        ... on RunConfigSchema {
            allConfigTypes {
                __typename
                key
                ... on CompositeConfigType {
                    __typename
                    fields {
                        __typename
                        name
                        configType {
                            key
                            __typename
                        }
                    }
                }
            }
        }
        ... on InvalidSubsetError {
            message
        }
        ... on PythonError {
            message
            stack
        }
    }
}
'''


def field_names_of(type_dict, typename):
    return {field_data['name'] for field_data in type_dict[typename]['fields']}


def types_dict_of_result(subset_result, top_key):
    return {
        type_data['name']: type_data for type_data in subset_result.data[top_key]['configTypes']
    }


class TestSolidSelections(LegacyReadonlyGraphQLContextTestMatrix):
    def test_csv_hello_world_pipeline_or_error_subset_wrong_solid_name(self, graphql_context):
        selector = infer_pipeline_selector(graphql_context, 'csv_hello_world', ['nope'])
        result = execute_dagster_graphql(
            graphql_context, SCHEMA_OR_ERROR_SUBSET_QUERY, {'selector': selector}
        )

        assert not result.errors
        assert result.data
        assert result.data['runConfigSchemaOrError']['__typename'] == 'InvalidSubsetError'
        assert '"nope" does not exist' in result.data['runConfigSchemaOrError']['message']

    @pytest.mark.skipif(sys.version_info.major < 3, reason='Exception cause only available on py3+')
    def test_pipeline_with_invalid_definition_error(self, graphql_context):
        selector = infer_pipeline_selector(
            graphql_context, 'pipeline_with_invalid_definition_error', ['fail_subset']
        )
        result = execute_dagster_graphql(
            graphql_context, SCHEMA_OR_ERROR_SUBSET_QUERY, {'selector': selector}
        )
        assert not result.errors
        assert result.data
        assert result.data['runConfigSchemaOrError']['__typename'] == 'InvalidSubsetError'

        assert re.match(
            (
                r'.*DagsterInvalidSubsetError[\s\S]*'
                r'add a input_hydration_config for the type "InputTypeWithoutHydration"'
            ),
            result.data['runConfigSchemaOrError']['message'],
        )



from dagster_graphql.test.utils import execute_dagster_graphql

from dagster.core.utils import make_new_run_id

from .execution_queries import EXECUTE_RUN_IN_PROCESS_QUERY
from .graphql_context_test_suite import ExecutingGraphQLContextTestMatrix
from .setup import (
    csv_hello_world,
    csv_hello_world_solids_config,
    main_repo_location_name,
    main_repo_name,
)


class TestStartPipelineForCreatedRunInHostedUserProcess(ExecutingGraphQLContextTestMatrix):
    def test_synchronously_execute_run_within_hosted_user_process(self, graphql_context):
        pipeline_run = graphql_context.instance.create_run_for_pipeline(
            pipeline_def=csv_hello_world, environment_dict=csv_hello_world_solids_config()
        )

        result = execute_dagster_graphql(
            graphql_context,
            EXECUTE_RUN_IN_PROCESS_QUERY,
            variables={
                'runId': pipeline_run.run_id,
                'repositoryLocationName': main_repo_location_name(),
                'repositoryName': main_repo_name(),
            },
        )
        assert result.data
        assert result.data['executeRunInProcess']['__typename'] == 'ExecuteRunInProcessSuccess'

    def test_synchronously_execute_run_within_hosted_user_process_not_found(self, graphql_context):
        run_id = make_new_run_id()
        result = execute_dagster_graphql(
            graphql_context,
            EXECUTE_RUN_IN_PROCESS_QUERY,
            variables={
                'runId': run_id,
                'repositoryLocationName': main_repo_location_name(),
                'repositoryName': main_repo_name(),
            },
        )

        assert result.data
        assert result.data['executeRunInProcess']['__typename'] == 'PipelineRunNotFoundError'



from graphql.execution.base import ResolveInfo

from dagster import check
from dagster.config.validate import validate_config_from_snap
from dagster.core.host_representation import RepresentedPipeline

from .external import get_external_pipeline_or_raise
from .utils import PipelineSelector, UserFacingGraphQLError, capture_dauphin_error


@capture_dauphin_error
def resolve_run_config_schema_or_error(graphene_info, selector, mode):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(selector, 'selector', PipelineSelector)
    check.opt_str_param(mode, 'mode')

    external_pipeline = get_external_pipeline_or_raise(graphene_info, selector)

    if mode is None:
        mode = external_pipeline.get_default_mode_name()

    if not external_pipeline.has_mode(mode):
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named('ModeNotFoundError')(mode=mode, selector=selector)
        )

    return graphene_info.schema.type_named('RunConfigSchema')(
        represented_pipeline=external_pipeline, mode=mode,
    )


@capture_dauphin_error
def resolve_is_run_config_valid(graphene_info, represented_pipeline, mode, environment_dict):
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.inst_param(represented_pipeline, 'represented_pipeline', RepresentedPipeline)
    check.str_param(mode, 'mode')
    check.dict_param(environment_dict, 'environment_dict', key_type=str)

    mode_def_snap = represented_pipeline.get_mode_def_snap(mode)

    if not mode_def_snap.root_config_key:
        # historical pipeline with unknown environment type. blindly pass validation
        return graphene_info.schema.type_named('PipelineConfigValidationValid')(
            represented_pipeline.name
        )

    validated_config = validate_config_from_snap(
        represented_pipeline.config_schema_snapshot, mode_def_snap.root_config_key, environment_dict
    )

    if not validated_config.success:
        raise UserFacingGraphQLError(
            graphene_info.schema.type_named('PipelineConfigValidationInvalid')(
                pipeline_name=represented_pipeline.name,
                errors=[
                    graphene_info.schema.type_named(
                        'PipelineConfigValidationError'
                    ).from_dagster_error(
                        represented_pipeline.config_schema_snapshot, err,
                    )
                    for err in validated_config.errors
                ],
            )
        )

    return graphene_info.schema.type_named('PipelineConfigValidationValid')(
        represented_pipeline.name
    )



import sys
from collections import namedtuple

from dagster import check
from dagster.core.host_representation import PipelineSelector
from dagster.utils.error import serializable_error_info_from_exc_info


def capture_dauphin_error(fn):
    def _fn(*args, **kwargs):
        from dagster_graphql.schema.errors import DauphinPythonError

        try:
            return fn(*args, **kwargs)
        except UserFacingGraphQLError as de_exception:
            return de_exception.dauphin_error
        except Exception:  # pylint: disable=broad-except
            return DauphinPythonError(serializable_error_info_from_exc_info(sys.exc_info()))

    return _fn


class UserFacingGraphQLError(Exception):
    def __init__(self, dauphin_error):
        self.dauphin_error = dauphin_error
        message = '[{cls}] {message}'.format(
            cls=dauphin_error.__class__.__name__,
            message=dauphin_error.message if hasattr(dauphin_error, 'message') else None,
        )
        super(UserFacingGraphQLError, self).__init__(message)


def pipeline_selector_from_graphql(context, data):
    from dagster_graphql.implementation.context import DagsterGraphQLContext

    check.inst_param(context, 'context', DagsterGraphQLContext)

    return PipelineSelector(
        location_name=data['repositoryLocationName'],
        repository_name=data['repositoryName'],
        pipeline_name=data['pipelineName'],
        solid_selection=data.get('solidSelection'),
    )


class ExecutionParams(
    namedtuple('_ExecutionParams', 'selector environment_dict mode execution_metadata step_keys',)
):
    def __new__(cls, selector, environment_dict, mode, execution_metadata, step_keys):
        check.dict_param(environment_dict, 'environment_dict', key_type=str)
        check.opt_list_param(step_keys, 'step_keys', of_type=str)

        return super(ExecutionParams, cls).__new__(
            cls,
            selector=check.inst_param(selector, 'selector', PipelineSelector),
            environment_dict=environment_dict,
            mode=check.str_param(mode, 'mode'),
            execution_metadata=check.inst_param(
                execution_metadata, 'execution_metadata', ExecutionMetadata
            ),
            step_keys=step_keys,
        )

    def to_graphql_input(self):
        return {
            'selector': self.selector.to_graphql_input(),
            'runConfigData': self.environment_dict,
            'mode': self.mode,
            'executionMetadata': self.execution_metadata.to_graphql_input(),
            'stepKeys': self.step_keys,
        }


class ExecutionMetadata(namedtuple('_ExecutionMetadata', 'run_id tags root_run_id parent_run_id')):
    def __new__(cls, run_id, tags, root_run_id=None, parent_run_id=None):
        return super(ExecutionMetadata, cls).__new__(
            cls,
            check.opt_str_param(run_id, 'run_id'),
            check.dict_param(tags, 'tags', key_type=str, value_type=str),
            check.opt_str_param(root_run_id, 'root_run_id'),
            check.opt_str_param(parent_run_id, 'parent_run_id'),
        )

    def to_graphql_input(self):
        return {
            'runId': self.run_id,
            'tags': [{'key': k, 'value': v} for k, v in self.tags.items()],
            'rootRunId': self.root_run_id,
            'parentRunId': self.parent_run_id,
        }



# -*- coding: utf-8 -*-
# snapshottest: v1 - https://goo.gl/zC4yUc
from __future__ import unicode_literals

from snapshottest import Snapshot

snapshots = Snapshot()

snapshots['TestRetryExecution.test_pipeline_reexecution_info_query[in_memory_instance_in_process_env] 1'] = [
    'sum_sq_solid.compute'
]

snapshots['TestRetryExecution.test_pipeline_reexecution_info_query[sqlite_with_sync_run_launcher_in_process_env] 1'] = [
    'sum_sq_solid.compute'
]

snapshots['TestRetryExecution.test_pipeline_reexecution_info_query[sqlite_with_cli_api_run_launcher_in_process_env] 1'] = [
    'sum_sq_solid.compute'
]



from dagster_graphql import dauphin
from dagster_graphql.implementation.fetch_schedules import get_schedule_yaml
from dagster_graphql.schema.errors import (
    DauphinPythonError,
    DauphinRepositoryNotFoundError,
    DauphinScheduleDefinitionNotFoundError,
)

from dagster import check
from dagster.core.host_representation import ExternalSchedule


class DapuphinScheduleDefinitionOrError(dauphin.Union):
    class Meta(object):
        name = 'ScheduleDefinitionOrError'
        types = ('ScheduleDefinition', DauphinScheduleDefinitionNotFoundError, DauphinPythonError)


class DauphinScheduleDefinitions(dauphin.ObjectType):
    class Meta(object):
        name = 'ScheduleDefinitions'

    results = dauphin.non_null_list('ScheduleDefinition')


class DauphinScheduleDefintionsOrError(dauphin.Union):
    class Meta(object):
        name = 'ScheduleDefinitionsOrError'
        types = (DauphinScheduleDefinitions, DauphinRepositoryNotFoundError, DauphinPythonError)


class DauphinScheduleDefinition(dauphin.ObjectType):
    class Meta(object):
        name = 'ScheduleDefinition'

    name = dauphin.NonNull(dauphin.String)
    cron_schedule = dauphin.NonNull(dauphin.String)
    pipeline_name = dauphin.NonNull(dauphin.String)
    solid_selection = dauphin.List(dauphin.String)
    mode = dauphin.NonNull(dauphin.String)
    schedule_state = dauphin.Field('ScheduleState')

    run_config_yaml = dauphin.Field(dauphin.String)
    partition_set = dauphin.Field('PartitionSet')

    def resolve_run_config_yaml(self, graphene_info):
        return get_schedule_yaml(graphene_info, self._external_schedule)

    def resolve_partition_set(self, graphene_info):
        if self._external_schedule.partition_set_name is None:
            return None

        repository = graphene_info.context.get_repository_location(
            self._external_schedule.handle.location_name
        ).get_repository(self._external_schedule.handle.repository_name)
        external_partition_set = repository.get_external_partition_set(
            self._external_schedule.partition_set_name
        )

        return graphene_info.schema.type_named('PartitionSet')(
            external_repository_handle=repository.handle,
            external_partition_set=external_partition_set,
        )

    def __init__(self, graphene_info, external_schedule):
        self._external_schedule = check.inst_param(
            external_schedule, 'external_schedule', ExternalSchedule
        )
        self._schedule_state = graphene_info.context.instance.get_schedule_state(
            self._external_schedule.get_origin_id()
        )

        super(DauphinScheduleDefinition, self).__init__(
            name=external_schedule.name,
            cron_schedule=external_schedule.cron_schedule,
            pipeline_name=external_schedule.pipeline_name,
            solid_selection=external_schedule.solid_selection,
            mode=external_schedule.mode,
            schedule_state=graphene_info.schema.type_named('ScheduleState')(
                graphene_info, self._schedule_state
            )
            if self._schedule_state
            else None,
        )



import yaml
from dagster_graphql import dauphin
from dagster_graphql.implementation.fetch_partition_sets import (
    get_partition_by_name,
    get_partition_config,
    get_partition_tags,
)
from dagster_graphql.implementation.fetch_runs import get_runs
from dagster_graphql.schema.errors import (
    DauphinPartitionSetNotFoundError,
    DauphinPipelineNotFoundError,
    DauphinPythonError,
)

from dagster import check
from dagster.core.host_representation import ExternalPartitionSet, RepositoryHandle
from dagster.core.storage.pipeline_run import PipelineRunsFilter


class DauphinPartition(dauphin.ObjectType):
    class Meta(object):
        name = 'Partition'

    name = dauphin.NonNull(dauphin.String)
    partition_set_name = dauphin.NonNull(dauphin.String)
    solid_selection = dauphin.List(dauphin.NonNull(dauphin.String))
    mode = dauphin.NonNull(dauphin.String)
    runConfigYaml = dauphin.NonNull(dauphin.String)
    tags = dauphin.non_null_list('PipelineTag')
    runs = dauphin.non_null_list('PipelineRun')

    def __init__(self, external_repository_handle, external_partition_set, partition_name):
        self._external_repository_handle = check.inst_param(
            external_repository_handle, 'external_respository_handle', RepositoryHandle
        )
        self._external_partition_set = check.inst_param(
            external_partition_set, 'external_partition_set', ExternalPartitionSet
        )
        self._partition_name = check.str_param(partition_name, 'partition_name')

        super(DauphinPartition, self).__init__(
            name=partition_name,
            partition_set_name=external_partition_set.name,
            solid_selection=external_partition_set.solid_selection,
            mode=external_partition_set.mode,
        )

    def resolve_runConfigYaml(self, _):
        run_config = get_partition_config(
            self._external_repository_handle,
            self._external_partition_set.name,
            self._partition_name,
        )

        if run_config is None:
            # TODO: surface user-facing error here
            # https://github.com/dagster-io/dagster/issues/2576
            return ''

        return yaml.dump(run_config, default_flow_style=False)

    def resolve_tags(self, graphene_info):
        tags = get_partition_tags(
            self._external_repository_handle,
            self._external_partition_set.name,
            self._partition_name,
        )

        if tags is None:
            # TODO: surface user-facing error here
            # https://github.com/dagster-io/dagster/issues/2576
            return []

        return [
            graphene_info.schema.type_named('PipelineTag')(key=key, value=value)
            for key, value in tags.items()
        ]

    def resolve_runs(self, graphene_info):
        runs_filter = PipelineRunsFilter(
            tags={
                'dagster/partition_set': self._external_partition_set.name,
                'dagster/partition': self._partition_name,
            }
        )
        return get_runs(graphene_info, runs_filter)


class DauphinPartitions(dauphin.ObjectType):
    class Meta(object):
        name = 'Partitions'

    results = dauphin.non_null_list('Partition')


class DauphinPartitionSet(dauphin.ObjectType):
    class Meta(object):
        name = 'PartitionSet'

    name = dauphin.NonNull(dauphin.String)
    pipeline_name = dauphin.NonNull(dauphin.String)
    solid_selection = dauphin.List(dauphin.NonNull(dauphin.String))
    mode = dauphin.NonNull(dauphin.String)
    partitions = dauphin.Field(
        dauphin.NonNull('Partitions'),
        cursor=dauphin.String(),
        limit=dauphin.Int(),
        reverse=dauphin.Boolean(),
    )
    partition = dauphin.Field('Partition', partition_name=dauphin.NonNull(dauphin.String))

    def __init__(self, external_repository_handle, external_partition_set):
        self._external_repository_handle = check.inst_param(
            external_repository_handle, 'external_respository_handle', RepositoryHandle
        )
        self._external_partition_set = check.inst_param(
            external_partition_set, 'external_partition_set', ExternalPartitionSet
        )

        super(DauphinPartitionSet, self).__init__(
            name=external_partition_set.name,
            pipeline_name=external_partition_set.pipeline_name,
            solid_selection=external_partition_set.solid_selection,
            mode=external_partition_set.mode,
        )

    def resolve_partitions(self, graphene_info, **kwargs):
        partition_names = self._external_partition_set.partition_names

        cursor = kwargs.get("cursor")
        limit = kwargs.get("limit")
        reverse = kwargs.get('reverse')

        start = 0
        end = len(partition_names)
        index = 0

        if cursor:
            index = next(
                (
                    idx
                    for (idx, partition_name) in enumerate(partition_names)
                    if partition_name == cursor
                ),
                None,
            )

            if reverse:
                end = index
            else:
                start = index + 1

        if limit:
            if reverse:
                start = end - limit
            else:
                end = start + limit

        partition_names = partition_names[start:end]

        return graphene_info.schema.type_named('Partitions')(
            results=[
                graphene_info.schema.type_named('Partition')(
                    external_partition_set=self._external_partition_set,
                    external_repository_handle=self._external_repository_handle,
                    partition_name=partition_name,
                )
                for partition_name in partition_names
            ]
        )

    def resolve_partition(self, graphene_info, partition_name):
        return get_partition_by_name(
            graphene_info,
            self._external_repository_handle,
            self._external_partition_set,
            partition_name,
        )


class DapuphinPartitionSetOrError(dauphin.Union):
    class Meta(object):
        name = 'PartitionSetOrError'
        types = ('PartitionSet', DauphinPartitionSetNotFoundError, DauphinPythonError)


class DauphinPartitionSets(dauphin.ObjectType):
    class Meta(object):
        name = 'PartitionSets'

    results = dauphin.non_null_list('PartitionSet')


class DauphinPartitionSetsOrError(dauphin.Union):
    class Meta(object):
        name = 'PartitionSetsOrError'
        types = (DauphinPartitionSets, DauphinPipelineNotFoundError, DauphinPythonError)



from dagster.core.execution.api import execute_run
from dagster.utils.hosted_user_process import recon_pipeline_from_origin

from ..utils import capture_dauphin_error
from .run_lifecycle import RunExecutionInfo, get_run_execution_info_for_created_run_or_error


@capture_dauphin_error
def execute_run_in_graphql_process(
    graphene_info, repository_location_name, repository_name, run_id
):
    '''This indirection is done on purpose to make the logic in the function
    below re-usable. The parent function is wrapped in @capture_dauphin_error, which makes it
    difficult to do exception handling.
    '''
    return _synchronously_execute_run_within_hosted_user_process(
        graphene_info, repository_location_name, repository_name, run_id
    )


def _synchronously_execute_run_within_hosted_user_process(
    graphene_info, repository_location_name, repository_name, run_id,
):
    run_info_or_error = get_run_execution_info_for_created_run_or_error(
        graphene_info, repository_location_name, repository_name, run_id
    )

    if not isinstance(run_info_or_error, RunExecutionInfo):
        # if it is not a success the return value is the dauphin error
        return run_info_or_error

    external_pipeline, pipeline_run = run_info_or_error
    recon_pipeline = recon_pipeline_from_origin(external_pipeline.get_origin())
    execute_run(recon_pipeline, pipeline_run, graphene_info.context.instance)
    return graphene_info.schema.type_named('ExecuteRunInProcessSuccess')(
        run=graphene_info.schema.type_named('PipelineRun')(pipeline_run)
    )



from dagster_graphql.implementation.utils import UserFacingGraphQLError
from dagster_graphql.schema.errors import DauphinInvalidSubsetError
from dagster_graphql.schema.pipelines import DauphinPipeline

from dagster import check
from dagster.core.host_representation import PipelineSelector, RepositoryLocation
from dagster.core.host_representation.external import ExternalPipeline
from dagster.core.instance import DagsterInstance


class DagsterGraphQLContext:
    def __init__(self, instance, locations, version=None):
        self._instance = check.inst_param(instance, 'instance', DagsterInstance)
        self._repository_locations = {}
        for loc in check.list_param(locations, 'locations', RepositoryLocation):
            check.invariant(
                self._repository_locations.get(loc.name) is None,
                'Can not have multiple locations with the same name, got multiple "{name}"'.format(
                    name=loc.name
                ),
            )
            self._repository_locations[loc.name] = loc
        self.version = version

    @property
    def instance(self):
        return self._instance

    @property
    def repository_locations(self):
        return list(self._repository_locations.values())

    def get_repository_location(self, name):
        return self._repository_locations[name]

    def has_repository_location(self, name):
        return name in self._repository_locations

    def get_subset_external_pipeline(self, selector):
        check.inst_param(selector, 'selector', PipelineSelector)
        # We have to grab the pipeline from the location instead of the repository directly
        # since we may have to request a subset we don't have in memory yet

        repository_location = self._repository_locations[selector.location_name]
        external_repository = repository_location.get_repository(selector.repository_name)

        subset_result = repository_location.get_subset_external_pipeline_result(selector)
        if not subset_result.success:
            error_info = subset_result.error
            raise UserFacingGraphQLError(
                DauphinInvalidSubsetError(
                    message="{message}{cause_message}".format(
                        message=error_info.message,
                        cause_message="\n{}".format(error_info.cause.message)
                        if error_info.cause
                        else "",
                    ),
                    pipeline=DauphinPipeline(self.get_full_external_pipeline(selector)),
                )
            )

        return ExternalPipeline(
            subset_result.external_pipeline_data, repository_handle=external_repository.handle,
        )

    def has_external_pipeline(self, selector):
        check.inst_param(selector, 'selector', PipelineSelector)
        if selector.location_name in self._repository_locations:
            loc = self._repository_locations[selector.location_name]
            if loc.has_repository(selector.repository_name):
                return loc.get_repository(selector.repository_name).has_external_pipeline(
                    selector.pipeline_name
                )

    def get_full_external_pipeline(self, selector):
        return (
            self._repository_locations[selector.location_name]
            .get_repository(selector.repository_name)
            .get_full_external_pipeline(selector.pipeline_name)
        )

    def get_external_execution_plan(
        self, external_pipeline, environment_dict, mode, step_keys_to_execute
    ):
        return self._repository_locations[
            external_pipeline.handle.location_name
        ].get_external_execution_plan(
            external_pipeline=external_pipeline,
            environment_dict=environment_dict,
            mode=mode,
            step_keys_to_execute=step_keys_to_execute,
        )

    def execute_plan(self, external_pipeline, environment_dict, pipeline_run, step_keys_to_execute):
        return self._repository_locations[external_pipeline.handle.location_name].execute_plan(
            instance=self.instance,
            external_pipeline=external_pipeline,
            environment_dict=environment_dict,
            pipeline_run=pipeline_run,
            step_keys_to_execute=step_keys_to_execute,
        )

    def execute_pipeline(self, external_pipeline, pipeline_run):
        return self._repository_locations[external_pipeline.handle.location_name].execute_pipeline(
            instance=self.instance, external_pipeline=external_pipeline, pipeline_run=pipeline_run
        )

    def drain_outstanding_executions(self):
        '''
        This ensures that any outstanding executions of runs are waited on.
        Useful for tests contexts when you want to ensure a started run
        has ended in order to verify its results.
        '''
        if self.instance.run_launcher:
            self.instance.run_launcher.join()



from collections import namedtuple

from dagster_graphql.schema.errors import DauphinPipelineConfigValidationInvalid
from graphql.execution.base import ResolveInfo

from dagster import check
from dagster.config.validate import validate_config_from_snap
from dagster.core.errors import DagsterInvalidConfigError
from dagster.core.events import EngineEventData
from dagster.core.storage.pipeline_run import PipelineRunStatus
from dagster.core.utils import make_new_run_id
from dagster.utils import merge_dicts
from dagster.utils.error import SerializableErrorInfo

from ..external import (
    ensure_valid_config,
    get_external_execution_plan_or_raise,
    get_external_pipeline_or_raise,
)
from ..resume_retry import compute_step_keys_to_execute
from ..utils import PipelineSelector


def create_valid_pipeline_run(graphene_info, external_pipeline, execution_params):
    ensure_valid_config(external_pipeline, execution_params.mode, execution_params.environment_dict)

    step_keys_to_execute = compute_step_keys_to_execute(
        graphene_info, external_pipeline, execution_params
    )

    external_execution_plan = get_external_execution_plan_or_raise(
        graphene_info=graphene_info,
        external_pipeline=external_pipeline,
        mode=execution_params.mode,
        environment_dict=execution_params.environment_dict,
        step_keys_to_execute=step_keys_to_execute,
    )

    return graphene_info.context.instance.create_run(
        pipeline_snapshot=external_pipeline.pipeline_snapshot,
        execution_plan_snapshot=external_execution_plan.execution_plan_snapshot,
        parent_pipeline_snapshot=external_pipeline.parent_pipeline_snapshot,
        pipeline_name=execution_params.selector.pipeline_name,
        run_id=execution_params.execution_metadata.run_id
        if execution_params.execution_metadata.run_id
        else make_new_run_id(),
        solids_to_execute=frozenset(execution_params.selector.solid_selection)
        if execution_params.selector.solid_selection
        else None,
        environment_dict=execution_params.environment_dict,
        mode=execution_params.mode,
        step_keys_to_execute=step_keys_to_execute,
        tags=merge_dicts(external_pipeline.tags, execution_params.execution_metadata.tags),
        root_run_id=execution_params.execution_metadata.root_run_id,
        parent_run_id=execution_params.execution_metadata.parent_run_id,
        status=PipelineRunStatus.NOT_STARTED,
    )


RunExecutionInfo = namedtuple('_RunExecutionInfo', 'external_pipeline pipeline_run')


def get_run_execution_info_for_created_run_or_error(
    graphene_info, repository_location_name, repository_name, run_id
):
    '''
    Previously created run could either be created in a different process *or*
    during the launchScheduledRun call where we want to have a record of
    a run the was created but have invalid configuration
    '''
    check.inst_param(graphene_info, 'graphene_info', ResolveInfo)
    check.str_param(repository_location_name, 'repository_location_name')
    check.str_param(repository_name, 'repository_name')
    check.str_param(run_id, 'run_id')

    instance = graphene_info.context.instance

    pipeline_run = instance.get_run_by_id(run_id)
    if not pipeline_run:
        return graphene_info.schema.type_named('PipelineRunNotFoundError')(run_id)

    external_pipeline = get_external_pipeline_or_raise(
        graphene_info,
        PipelineSelector(
            location_name=repository_location_name,
            repository_name=repository_name,
            pipeline_name=pipeline_run.pipeline_name,
            solid_selection=list(pipeline_run.solids_to_execute)
            if pipeline_run.solids_to_execute
            else None,
        ),
    )

    validated_config = validate_config_from_snap(
        external_pipeline.config_schema_snapshot,
        external_pipeline.root_config_key_for_mode(pipeline_run.mode),
        pipeline_run.environment_dict,
    )

    if not validated_config.success:
        # If the config is invalid, we construct a DagsterInvalidConfigError exception and
        # insert it into the event log. We also return a PipelineConfigValidationInvalid user facing
        # graphql error.

        # We currently re-use the engine events machinery to add the error to the event log, but
        # may need to create a new event type and instance method to handle these errors.
        invalid_config_exception = DagsterInvalidConfigError(
            'Error in config for pipeline {}'.format(external_pipeline.name),
            validated_config.errors,
            pipeline_run.environment_dict,
        )

        instance.report_engine_event(
            str(invalid_config_exception.message),
            pipeline_run,
            EngineEventData.engine_error(
                SerializableErrorInfo(
                    invalid_config_exception.message,
                    [],
                    DagsterInvalidConfigError.__class__.__name__,
                    None,
                )
            ),
        )

        instance.report_run_failed(pipeline_run)

        return DauphinPipelineConfigValidationInvalid.for_validation_errors(
            external_pipeline, validated_config.errors
        )

    return RunExecutionInfo(external_pipeline, pipeline_run)



from dagster_graphql.test.utils import execute_dagster_graphql, infer_pipeline_selector

from .composites_query import (
    COMPOSITES_QUERY,
    COMPOSITES_QUERY_NESTED_DEPENDS_ON_DEPENDS_BY_CORE,
    NESTED_INPUT_DEPENDS_ON,
    NESTED_OUTPUT_DEPENDED_BY,
    PARENT_ID_QUERY,
    SOLID_ID_QUERY,
)
from .graphql_context_test_suite import ReadonlyGraphQLContextTestMatrix

# 10 total solids in the composite pipeline:
#
# (+1) \
#       (+2)
# (+1) /    \
#            (+4)
# (+1) \    /
#       (+2)
# (+1) /
#
#       (/2)
#           \
#            (/4)
#           /
#       (/2)


# this only needs readonly variants since they never execute anything
class TestComposites(ReadonlyGraphQLContextTestMatrix):
    def test_composites(self, graphql_context, snapshot):
        selector = infer_pipeline_selector(graphql_context, "composites_pipeline")
        result = execute_dagster_graphql(graphql_context, COMPOSITES_QUERY, {'selector': selector})
        handle_map = {}
        for obj in result.data['pipelineOrError']['solidHandles']:
            handle_map[obj['handleID']] = obj['solid']

        assert len(handle_map) == 10

        snapshot.assert_match(result.data)

    def test_parent_id_arg(self, graphql_context):
        selector = infer_pipeline_selector(graphql_context, "composites_pipeline")
        result = execute_dagster_graphql(graphql_context, PARENT_ID_QUERY, {'selector': selector})
        assert len(result.data['pipelineOrError']['solidHandles']) == 10

        result = execute_dagster_graphql(
            graphql_context, PARENT_ID_QUERY, {'selector': selector, 'parentHandleID': ''}
        )
        assert len(result.data['pipelineOrError']['solidHandles']) == 2

        result = execute_dagster_graphql(
            graphql_context, PARENT_ID_QUERY, {'selector': selector, 'parentHandleID': 'add_four'}
        )
        assert len(result.data['pipelineOrError']['solidHandles']) == 2

        result = execute_dagster_graphql(
            graphql_context,
            PARENT_ID_QUERY,
            {'selector': selector, 'parentHandleID': 'add_four.adder_1'},
        )
        assert len(result.data['pipelineOrError']['solidHandles']) == 2

        result = execute_dagster_graphql(
            graphql_context,
            PARENT_ID_QUERY,
            {'selector': selector, 'parentHandleID': 'add_four.doot'},
        )
        assert len(result.data['pipelineOrError']['solidHandles']) == 0

    def test_solid_id(self, graphql_context):
        selector = infer_pipeline_selector(graphql_context, "composites_pipeline")
        result = execute_dagster_graphql(
            graphql_context, SOLID_ID_QUERY, {'selector': selector, 'id': 'add_four'}
        )
        assert result.data['pipelineOrError']['solidHandle']['handleID'] == 'add_four'

        result = execute_dagster_graphql(
            graphql_context,
            SOLID_ID_QUERY,
            {'selector': selector, 'id': 'add_four.adder_1.adder_1'},
        )
        assert (
            result.data['pipelineOrError']['solidHandle']['handleID'] == 'add_four.adder_1.adder_1'
        )

        result = execute_dagster_graphql(
            graphql_context, SOLID_ID_QUERY, {'selector': selector, 'id': 'bonkahog'}
        )
        assert result.data['pipelineOrError']['solidHandle'] == None

    def test_recurse_composites_depends(self, graphql_context):
        selector = infer_pipeline_selector(graphql_context, "composites_pipeline")
        execute_dagster_graphql(
            graphql_context,
            COMPOSITES_QUERY_NESTED_DEPENDS_ON_DEPENDS_BY_CORE + NESTED_INPUT_DEPENDS_ON,
            {'selector': selector},
        )

        execute_dagster_graphql(
            graphql_context,
            COMPOSITES_QUERY_NESTED_DEPENDS_ON_DEPENDS_BY_CORE + NESTED_OUTPUT_DEPENDED_BY,
            {'selector': selector},
        )



from dagster_graphql.test.utils import execute_dagster_graphql, infer_repository_selector

from .graphql_context_test_suite import LegacyReadonlyGraphQLContextTestMatrix

GET_PARTITION_SETS_FOR_PIPELINE_QUERY = '''
    query PartitionSetsQuery($repositorySelector: RepositorySelector!, $pipelineName: String!) {
        partitionSetsOrError(repositorySelector: $repositorySelector, pipelineName: $pipelineName) {
            __typename
            ...on PartitionSets {
                results {
                    name
                    pipelineName
                    solidSelection
                    mode
                }
            }
            ... on PythonError {
                message
                stack
            }
            ...on PipelineNotFoundError {
                message
            }
        }
    }
'''

GET_PARTITION_SET_QUERY = '''
    query PartitionSetQuery($repositorySelector: RepositorySelector!, $partitionSetName: String!) {
        partitionSetOrError(repositorySelector: $repositorySelector, partitionSetName: $partitionSetName) {
            __typename
            ... on PythonError {
                message
                stack
            }
            ...on PartitionSet {
                name
                pipelineName
                solidSelection
                mode
                partitions {
                    results {
                        name
                    }
                }
            }
        }
    }
'''


class TestPartitionSets(LegacyReadonlyGraphQLContextTestMatrix):
    def test_get_partition_sets_for_pipeline(self, graphql_context, snapshot):
        selector = infer_repository_selector(graphql_context)
        result = execute_dagster_graphql(
            graphql_context,
            GET_PARTITION_SETS_FOR_PIPELINE_QUERY,
            variables={'repositorySelector': selector, 'pipelineName': 'no_config_pipeline'},
        )

        assert result.data
        snapshot.assert_match(result.data)

        invalid_pipeline_result = execute_dagster_graphql(
            graphql_context,
            GET_PARTITION_SETS_FOR_PIPELINE_QUERY,
            variables={'repositorySelector': selector, 'pipelineName': 'invalid_pipeline'},
        )

        assert invalid_pipeline_result.data
        snapshot.assert_match(invalid_pipeline_result.data)

    def test_get_partition_set(self, graphql_context, snapshot):
        selector = infer_repository_selector(graphql_context)
        result = execute_dagster_graphql(
            graphql_context,
            GET_PARTITION_SET_QUERY,
            variables={'partitionSetName': 'integer_partition', 'repositorySelector': selector},
        )

        assert result.data
        snapshot.assert_match(result.data)

        invalid_partition_set_result = execute_dagster_graphql(
            graphql_context,
            GET_PARTITION_SET_QUERY,
            variables={'partitionSetName': 'invalid_partition', 'repositorySelector': selector},
        )

        print(invalid_partition_set_result.data)
        assert (
            invalid_partition_set_result.data['partitionSetOrError']['__typename']
            == 'PartitionSetNotFoundError'
        )
        assert invalid_partition_set_result.data

        snapshot.assert_match(invalid_partition_set_result.data)



FRAGMENTS = '''
fragment metadataEntryFragment on EventMetadataEntry {
    label
    description
    ... on EventPathMetadataEntry {
        path
    }
    ... on EventJsonMetadataEntry {
        jsonString
    }
    ... on EventTextMetadataEntry {
        text
    }
    ... on EventUrlMetadataEntry {
        url
    }
    ... on EventPythonArtifactMetadataEntry {
        module
        name
    }
    ... on EventFloatMetadataEntry {
        value
    }
}

fragment stepEventFragment on StepEvent {
    ... on ExecutionStepStartEvent {
        step { key kind }
    }
    ... on ExecutionStepSuccessEvent {
        step { key }
    }
    ... on ExecutionStepFailureEvent {
        step { key kind }
        error {
            message
        }
        failureMetadata {
            label
            description
            metadataEntries {
                ...metadataEntryFragment
            }
        }
    }
    ... on ExecutionStepSkippedEvent {
        step { key }
    }
    ... on ExecutionStepInputEvent {
        step { key kind }
        inputName
        typeCheck {
            label
            description
            metadataEntries {
                ...metadataEntryFragment
            }
        }
    }
    ... on ExecutionStepOutputEvent {
        step { key kind }
        outputName
        typeCheck {
            label
            description
            metadataEntries {
                ...metadataEntryFragment
            }
        }
    }
    ... on StepExpectationResultEvent {
        expectationResult {
            success
            label
            description
            metadataEntries {
                ...metadataEntryFragment
            }
        }
    }
    ... on ObjectStoreOperationEvent {
        step { key }
        operationResult {
            op
            metadataEntries {
                ...metadataEntryFragment
            }
        }
    }
}
'''

LAUNCH_PIPELINE_EXECUTION_RESULT_FRAGMENT = '''
    fragment launchPipelineExecutionResultFragment on LaunchPipelineExecutionResult {
        __typename
        ... on InvalidStepError {
            invalidStepKey
        }
        ... on InvalidOutputError {
            stepKey
            invalidOutputName
        }
        ... on PipelineConfigValidationInvalid {
            pipelineName
            errors {
                __typename
                message
                path
                reason
            }
        }
        ... on PipelineNotFoundError {
            message
            pipelineName
        }
        ... on PythonError {
            message
            stack
            cause {
                message
                stack
            }
        }
        ... on LaunchPipelineRunSuccess {
            run {
                runId
                pipeline {
                    ... on PipelineReference {
                        name
                    }
                }
                tags {
                    key
                    value
                }
            }
        }
    }
'''

LAUNCH_PIPELINE_EXECUTION_RESULT_FRAGMENT = '''
    fragment launchPipelineExecutionResultFragment on LaunchPipelineExecutionResult {
        __typename
        ... on LaunchPipelineRunSuccess {
            run {
                runId
                pipeline { ...on PipelineReference { name } }
                tags {
                    key
                    value
                }
            }
        }
        ... on PipelineConfigValidationInvalid {
            pipelineName
            errors { message }
        }
        ... on PipelineNotFoundError {
            pipelineName
        }
        ... on PythonError {
            message
            stack
            cause {
                message
                stack
            }
        }
    }
'''


EXECUTE_RUN_IN_PROCESS_RESULT_FRAGMENT = '''
    fragment executeRunInProcessResultFragment on ExecuteRunInProcessResult {
        __typename
        ... on ExecuteRunInProcessSuccess {
            run {
                runId
                pipeline { ...on PipelineReference { name } }
                tags {
                    key
                    value
                }
            }
        }
        ... on PipelineConfigValidationInvalid {
            pipelineName
            errors { message }
        }
        ... on PipelineNotFoundError {
            pipelineName
        }
        ... on PythonError {
            message
            stack
            cause {
                message
                stack
            }
        }
        ... on PipelineRunNotFoundError {
            message
        }
    }
'''

LAUNCH_PIPELINE_REEXECUTION_RESULT_FRAGMENT = '''
    fragment launchPipelineReexecutionResultFragment on LaunchPipelineReexecutionResult {
        __typename
        ... on LaunchPipelineRunSuccess {
            run {
                runId
                pipeline { ...on PipelineReference { name } }
                tags {
                    key
                    value
                }
            }
        }
        ... on PipelineConfigValidationInvalid {
            pipelineName
            errors { message }
        }
        ... on PipelineNotFoundError {
            pipelineName
        }
        ... on PythonError {
            message
            stack
            cause {
                message
                stack
            }
        }
    }
'''

LAUNCH_PIPELINE_EXECUTION_QUERY = (
    LAUNCH_PIPELINE_EXECUTION_RESULT_FRAGMENT
    + '''

mutation (
    $executionParams: ExecutionParams!
) {
    launchPipelineExecution(
        executionParams: $executionParams
    ) {
        ...launchPipelineExecutionResultFragment
    }
}
'''
)

EXECUTE_RUN_IN_PROCESS_QUERY = (
    EXECUTE_RUN_IN_PROCESS_RESULT_FRAGMENT
    + '''

mutation (
    $repositoryLocationName: String!
    $repositoryName: String!
    $runId: String!
) {
    executeRunInProcess(
        repositoryLocationName: $repositoryLocationName
        repositoryName: $repositoryName
        runId: $runId
    ) {
        ...executeRunInProcessResultFragment
    }
}
'''
)


LAUNCH_PIPELINE_REEXECUTION_QUERY = (
    LAUNCH_PIPELINE_REEXECUTION_RESULT_FRAGMENT
    + '''

mutation (
    $executionParams: ExecutionParams!
) {
    launchPipelineReexecution(
        executionParams: $executionParams
    ) {
        ...launchPipelineReexecutionResultFragment
    }
}
'''
)


LAUNCH_PIPELINE_REEXECUTION_SNAPSHOT_QUERY = '''
mutation (
    $executionParams: ExecutionParams!
) {
    launchPipelineReexecution(
        executionParams: $executionParams
    ) {
        __typename
        ... on LaunchPipelineRunSuccess {
            run {
                pipeline { ...on PipelineReference { name } }
                tags {
                    key
                    value
                }
                rootRunId
                parentRunId
            }
        }
        ... on PipelineConfigValidationInvalid {
            pipelineName
            errors { message }
        }
        ... on PipelineNotFoundError {
            pipelineName
        }
        ... on InvalidStepError {
            invalidStepKey
        }
        ... on InvalidOutputError {
            stepKey
            invalidOutputName
        }
    }
}
'''
LAUNCH_PIPELINE_EXECUTION_SNAPSHOT_FRIENDLY = '''
mutation(
  $executionParams: ExecutionParams!
) {
  launchPipelineExecution(
    executionParams: $executionParams,
  ) {
    __typename
    ... on InvalidStepError {
      invalidStepKey
    }
    ... on InvalidOutputError {
      stepKey
      invalidOutputName
    }
    ... on PipelineConfigValidationInvalid {
      pipelineName
      errors {
        __typename
        message
        path
        reason
      }
    }
    ... on PipelineNotFoundError {
      message
      pipelineName
    }
    ... on PythonError {
      message
      stack
    }
    ... on LaunchPipelineRunSuccess {
      run {
        status
        pipeline {
          name
        }
        mode
      }
    }
  }
}
'''
SUBSCRIPTION_QUERY = (
    FRAGMENTS
    + '''
subscription subscribeTest($runId: ID!) {
    pipelineRunLogs(runId: $runId) {
        __typename
        ... on PipelineRunLogsSubscriptionSuccess {
            run {
                runId
            },
            messages {
                __typename
                ...stepEventFragment

                ... on MessageEvent {
                    message
                    step { key solidHandleID }
                    level
                }

                # only include here because unstable between runs
                ... on StepMaterializationEvent {
                    materialization {
                        label
                        description
                        metadataEntries {
                            __typename
                            ...metadataEntryFragment
                        }
                    }
                }
                ... on ExecutionStepFailureEvent {
                    step { key kind }
                    error {
                        message
                        stack
                    }
                }
            }
        }
        ... on PipelineRunLogsSubscriptionFailure {
            missingRunId
            message
        }
    }
}
'''
)


PIPELINE_REEXECUTION_INFO_QUERY = '''
query ReexecutionInfoQuery($runId: ID!) {
  pipelineRunOrError(runId: $runId) {
    __typename
    ... on PipelineRun {
        stepKeysToExecute
      }
    }
  }
'''



import sys

import pytest
from dagster_graphql.test.utils import execute_dagster_graphql, infer_pipeline_selector

from .graphql_context_test_suite import LegacyReadonlyGraphQLContextTestMatrix
from .setup import csv_hello_world_solids_config

RUN_CONFIG_SCHEMA_QUERY = '''
query($selector: PipelineSelector! $mode: String!)
{
  runConfigSchemaOrError(selector: $selector, mode: $mode){
    __typename
    ... on RunConfigSchema {
      rootConfigType {
        key
      }
      allConfigTypes {
        key
      }
    }
  }
}
'''

RUN_CONFIG_SCHEMA_CONFIG_TYPE_QUERY = '''
query($selector: PipelineSelector! $mode: String! $configTypeName: String!)
{
  runConfigSchemaOrError(selector: $selector, mode: $mode){
    __typename
    ... on RunConfigSchema {
      configTypeOrError(configTypeName: $configTypeName) {
        __typename
        ... on EnumConfigType {
          name
        }
        ... on RegularConfigType {
          name
        }
        ... on CompositeConfigType {
          name
        }
      }
    }
  }
}
'''


RUN_CONFIG_SCHEMA_CONFIG_VALIDATION_QUERY = '''
query PipelineQuery(
    $runConfigData: RunConfigData,
    $selector: PipelineSelector!,
    $mode: String!
) {
  runConfigSchemaOrError(selector: $selector mode: $mode) {
    ... on RunConfigSchema {
      isRunConfigValid(runConfigData: $runConfigData) {
        __typename
        ... on PipelineConfigValidationValid {
            pipelineName
        }
        ... on PipelineConfigValidationInvalid {
            pipelineName
            errors {
                __typename
                ... on RuntimeMismatchConfigError {
                    valueRep
                }
                ... on MissingFieldConfigError {
                    field { name }
                }
                ... on MissingFieldsConfigError {
                    fields { name }
                }
                ... on FieldNotDefinedConfigError {
                    fieldName
                }
                ... on FieldsNotDefinedConfigError {
                    fieldNames
                }
                ... on SelectorTypeConfigError {
                    incomingFields
                }
                message
                reason
                stack {
                    entries {
                        __typename
                        ... on EvaluationStackPathEntry {
                            fieldName
                        }
                        ... on EvaluationStackListItemEntry {
                            listIndex
                        }
                    }
                }
            }
        }
        ... on PipelineNotFoundError {
            pipelineName
        }
      }
    }
  }
}
'''


class TestEnvironmentSchema(LegacyReadonlyGraphQLContextTestMatrix):
    def test_successful_run_config_schema(self, graphql_context):
        selector = infer_pipeline_selector(graphql_context, 'multi_mode_with_resources')
        result = execute_dagster_graphql(
            graphql_context,
            RUN_CONFIG_SCHEMA_QUERY,
            variables={'selector': selector, 'mode': 'add_mode',},
        )
        assert result.data['runConfigSchemaOrError']['__typename'] == 'RunConfigSchema'

    def test_run_config_schema_pipeline_not_found(self, graphql_context):
        selector = infer_pipeline_selector(graphql_context, 'jkdjfkdjfd')
        result = execute_dagster_graphql(
            graphql_context,
            RUN_CONFIG_SCHEMA_QUERY,
            variables={'selector': selector, 'mode': 'add_mode'},
        )
        assert result.data['runConfigSchemaOrError']['__typename'] == 'PipelineNotFoundError'

    def test_run_config_schema_solid_not_found(self, graphql_context):
        selector = infer_pipeline_selector(
            graphql_context, 'multi_mode_with_resources', ['kdjfkdj']
        )
        result = execute_dagster_graphql(
            graphql_context,
            RUN_CONFIG_SCHEMA_QUERY,
            variables={'selector': selector, 'mode': 'add_mode',},
        )
        assert result.data['runConfigSchemaOrError']['__typename'] == 'InvalidSubsetError'

    def test_run_config_schema_mode_not_found(self, graphql_context):
        selector = infer_pipeline_selector(graphql_context, 'multi_mode_with_resources')
        result = execute_dagster_graphql(
            graphql_context,
            RUN_CONFIG_SCHEMA_QUERY,
            variables={'selector': selector, 'mode': 'kdjfdk'},
        )
        assert result.data['runConfigSchemaOrError']['__typename'] == 'ModeNotFoundError'

    def test_basic_valid_config_on_run_config_schema(self, graphql_context, snapshot):
        selector = infer_pipeline_selector(graphql_context, 'csv_hello_world')
        result = execute_dagster_graphql(
            graphql_context,
            RUN_CONFIG_SCHEMA_CONFIG_VALIDATION_QUERY,
            variables={
                'selector': selector,
                'mode': 'default',
                'runConfigData': csv_hello_world_solids_config(),
            },
        )

        assert not result.errors
        assert result.data
        assert (
            result.data['runConfigSchemaOrError']['isRunConfigValid']['__typename']
            == 'PipelineConfigValidationValid'
        )
        snapshot.assert_match(result.data)

    @pytest.mark.skipif(sys.version_info < (3,), reason="Snapshot not working correctly on py2")
    def test_basic_invalid_config_on_run_config_schema(self, graphql_context, snapshot):
        selector = infer_pipeline_selector(graphql_context, 'csv_hello_world')
        result = execute_dagster_graphql(
            graphql_context,
            RUN_CONFIG_SCHEMA_CONFIG_VALIDATION_QUERY,
            variables={
                'selector': selector,
                'mode': 'default',
                'runConfigData': {'nope': 'kdjfd'},
            },
        )

        assert not result.errors
        assert result.data
        assert (
            result.data['runConfigSchemaOrError']['isRunConfigValid']['__typename']
            == 'PipelineConfigValidationInvalid'
        )
        snapshot.assert_match(result.data)
